<?php
/**
 * This file shows Google/Authy authenticator frontend.
 *
 * @package miniorange-2-factor-authentication/views/twofa/test
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Function to show Google/Authy authenticator frontend.
 *
 * @param object $user User object.
 * @param string $method 2-factor method of user.
 * @return void
 */
function mo2f_test_google_authy_authenticator( $user, $method ) {

	?>
		<h3>
			<?php
			printf(
				/* translators: %s: Name of the 2fa method */
				esc_html__( 'Test %s', 'miniorange-2-factor-authentication' ),
				esc_html( $method )
			);
			?>
			</h3>
		<hr>
	<p>
	<?php
	printf(
		/* translators: %s: Name of the 2fa method */
		esc_html__( 'Enter the verification code from the configured account in your %s app.', 'miniorange-2-factor-authentication' ),
		esc_html( $method )
	);
	?>
				</p>

	<form name="f" method="post" action="">
		<input type="hidden" name="option" value="mo2f_validate_google_authy_test"/>
		<input type="hidden" name="mo2f_validate_google_authy_test_nonce"
						value="<?php echo esc_attr( wp_create_nonce( 'mo2f-validate-google-authy-test-nonce' ) ); ?>"/>

		<input class="mo2f_table_textbox" style="width:200px;" autofocus="true" type="text" name="otp_token" required
			placeholder="<?php esc_attr_e( 'Enter OTP', 'miniorange-2-factor-authentication' ); ?>" style="width:100%;"/>
		<br><br>
			<input type="button" name="back" id="go_back" class="button button-primary button-large"
				value="<?php esc_attr_e( 'Back', 'miniorange-2-factor-authentication' ); ?>"/>
		<input type="submit" name="validate" id="validate" class="button button-primary button-large"
			value="<?php esc_attr_e( 'Submit', 'miniorange-2-factor-authentication' ); ?>"/>

	</form>
	<form name="f" method="post" action="" id="mo2f_go_back_form">
		<input type="hidden" name="option" value="mo2f_go_back"/>
		<input type="hidden" name="mo2f_go_back_nonce"
						value="<?php echo esc_attr( wp_create_nonce( 'mo2f-go-back-nonce' ) ); ?>"/>
	</form>
	<script>
		jQuery('#go_back').click(function () {
			jQuery('#mo2f_go_back_form').submit();
		});
	</script>

	<?php
}

/**
 * Function to show Test OTP over Email frontend.
 *
 * @param object $user User object.
 * @param string $method 2-factor method of the user.
 * @return void
 */
function mo2f_test_otp_over_email( $user, $method ) {

	?>
		<h3>
			<?php
				printf(
					/* translators: %s: Name of the 2fa method */
					esc_html__( 'Test %s.', 'miniorange-2-factor-authentication' ),
					esc_html( $method )
				);
			?>
			</h3>
		<h4> Remaining Email Transaction: <?php echo intval( esc_html( ( MoWpnsUtility::get_mo2f_db_option( 'cmVtYWluaW5nT1RQ', 'site_option' ) ) ) ); ?> </h4> 	
		<hr>
	<p><?php esc_html_e( 'Enter the one time passcode sent to your registered email id.', 'miniorange-2-factor-authentication' ); ?></p>

	<form name="f" method="post" action="">
		<input type="hidden" name="option" value="mo2f_validate_otp_over_email"/>
		<input type="hidden" name="mo2f_validate_otp_over_email_test_nonce"
						value="<?php echo esc_attr( wp_create_nonce( 'mo2f-validate-otp-over-email-test-nonce' ) ); ?>"/>

		<input class="mo2f_table_textbox" style="width:200px;" autofocus="true" type="text" name="otp_token" required placeholder="<?php esc_attr_e( 'Enter OTP', 'miniorange-2-factor-authentication' ); ?>" style="width:95%;"/>
		&nbsp;<a href="#mo2f_resendemaillink" id="mo2f_resendemaillink"><?php esc_html_e( 'Resend OTP?', 'miniorange-2-factor-authentication' ); ?></a>
			<br><br>
			<input type="button" name="back" id="go_back" class="button button-primary button-large"
				value="<?php esc_attr_e( 'Back', 'miniorange-2-factor-authentication' ); ?>"/>
		<input type="submit" name="validate" id="validate" class="button button-primary button-large"
				value="<?php esc_attr_e( 'Submit', 'miniorange-2-factor-authentication' ); ?>"/>

	</form>
	<form name="f" method="post" action="" id="mo2f_go_back_form">
		<input type="hidden" name="option" value="mo2f_go_back"/>
		<input type="hidden" name="mo2f_go_back_nonce"
						value="<?php echo esc_attr( wp_create_nonce( 'mo2f-go-back-nonce' ) ); ?>"/>
	</form>
	<script>
		jQuery('#go_back').click(function () {
			jQuery('#mo2f_go_back_form').submit();
		});
		jQuery('a[href=\"#mo2f_resendemaillink\"]').click(function(){
			var email   = '<?php echo esc_js( get_user_meta( $user->ID, 'tempRegEmail', true ) ); ?>';
			var nonce   = '<?php echo esc_js( wp_create_nonce( 'mo-two-factor-ajax-nonce' ) ); ?>';
			var data = {
			'action'                        : 'mo_two_factor_ajax',
			'mo_2f_two_factor_ajax'         : 'mo2f_save_email_verification', 
			'nonce'                         : nonce,
			'email'                         : email,
			'current_method'                : 'OTPOverEmail'
			};
			jQuery.post(ajaxurl, data, function(response) {    
				var response = response.replace(/\s+/g,' ').trim();
				if(response=="settingsSaved")
				{
					success_msg("OTP has been resent to the email " + email + ".");
				}
				else if(response == "NonceDidNotMatch")
				{   
					error_msg("An unknown error has occured.");
				}else if(response=="USER_LIMIT_EXCEEDED"){
					error_msg(" Your limit of 3 users has exceeded. Please upgrade to premium plans to setup 2FA for more users");
				}
				else if (response == "smtpnotset"){
					error_msg(" Please set up SMTP for your website to receive emails and prevent the accidental lock out");
				}
				else
				{
					error_msg(" Invalid Email.");
				}
			});

		});
	</script>

	<?php
}
